/***************************************************************************
 *   Copyright (C) 2006-2015 Helge Kortendieck <alan@kasmithee.de>         *
 *   Handling the ImageMagick processes                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "magick.h"


magick::magick(QObject *parent) : QObject(parent)
{
	iProc = 0;
}

magick::~magick()
{
}

void magick::magickChk()					// check for ImageMagick convert command
{
	iChk = 0;
	mag = new QProcess(this);
	sCmd = "convert";
	slArgs.clear();
	mag->start(sCmd, slArgs);
	if (mag->error() == QProcess::FailedToStart) {	// ImageMagick convert could not be found
		iChk = 1;
	}
	emit sigChkEnd(iChk);
}

void magick::magickAppendPrep(const QString& sD, const QStringList& slD, bool bD, bool bD2)	// prepare qprocess: convert append, concatenate every 2 images into 1
{
	sMTmpDir = sD;
	slToProcess = slD;
	bPJ = bD;									// bool create pdf (true) or jpg (false)
	iProc = 1;									// process type: 0=none, 1=append, 2=convert
	bApH = bD2;									// bool append jpgs horizontally: true=horizontally, false=vertically
	iAppendCnt = slToProcess.count() / 2;
	iAppendIter = 0;
	slResult.clear();							// list with images after appending 1 to 1, left to right or top to bottom
	procPrep();
}

QString magick::magickAppendCreatePicName()	// create append picture name padding
{
	if (iAppendIter < 9) {
		sPad = "0000";
	} else if (iAppendIter < 99) {
		sPad = "000";
	} else if (iAppendIter < 999) {
		sPad = "00";
	} else if (iAppendIter < 9999) {
		sPad = "0";
	}
	return sMTmpDir + "r" + sPad + QString::number(iAppendIter + 1) + "." + sSuffix;
}

void magick::magickToPdfPrep(const QString& sD, const QStringList& slD, const QString& sD2, const QString& sD3, const QString& sD4)	// prepare converting images into one pdf
{
	sMTmpDir = sD;
	slToProcess = slD;
	sCompress = sD2;							// compress type
	sQuality = sD3;								// quality setting
	sMOutputFile = sD4;							// full path n name of output file
	QFileInfo fi(slToProcess.at(0));
		sSuffix = fi.suffix();
	iProc = 2;
	procPrep();
}

void magick::procPrep()						// prepare qprocess
{
//cout << "magick::procPrep() reached " << endl;      //  *******************  debug  !!!!!!!!!!
	slArgs.clear();

// **  appending 2 pics into 1, bApH=true: left & right, bApH=false: top & bottom
	if (iProc == 1) {
		if (bApH == true){
			slArgs.append("+append");							// argument append horizontally or vertically
		} else {
			slArgs.append("-append");
		}

		slArgs.append(slToProcess.at(iAppendIter * 2));			// argument pic 1
		slArgs.append(slToProcess.at(iAppendIter * 2 + 1));		// argument pic 2
		QFileInfo fi(slToProcess.at(iAppendIter * 2));
			sSuffix = fi.suffix();

		if (bPJ == true) {										// just concatenating 2 pics into 1
			sMOutputFile = sMTmpDir + QString::number(iAppendIter) + "." + sSuffix;	// set the output file path/name for this append run
			slArgs.append(sMOutputFile);						// argument resulting pic, full path n name came overloaded as tmp dir
			slResult.append(sMOutputFile);
		} else {												// concatentating pic(s) before creating pdf, we need to create file names here!
			slArgs.append(sMTmpDir);							// argument resulting pic, full path n name came overloaded as tmp dir
			slResult.append(sMTmpDir);
		}

// **  convert image(s) into 1 pdf
	} else if (iProc == 2) {
		if (sCompress.isEmpty() == false) {
			slArgs.append("-compress");							// argument compress images with type ...
			slArgs.append(sCompress);							// argument compress type
		}
		if (sQuality != "-1") {
			slArgs.append("-quality");							// argument compress type quality
			slArgs.append(sQuality);							// argument quality setting
		}
		if (sMTmpDir != "") {
			slArgs.append(sMTmpDir + "*." + sSuffix);			// argument temporary directory - take everything from here with ending 'suffix'
		} else {
			for (iD = 0; iD < slToProcess.count(); ++iD) {		// arguments: add files to include as arguments, one after another (pics were not concatenated at first)
				slArgs.append(slToProcess.at(iD));
			}
		}
		slArgs.append(sMOutputFile);							// argument output file, full path and name (usually not in tmp dir)

	} else {
		return;
	}

    procStart();
}


void magick::procStart()					// run convert process
{
	bErr = false;
	mag = new QProcess(this);
//	connect(mag, SIGNAL(readyReadStandardOutput()), this, SLOT(procReadStdOut()));
	connect(mag, SIGNAL(readyReadStandardError()), this, SLOT(procReadStdErr()));
	connect(mag, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(procExit(int, QProcess::ExitStatus)));
	mag->start("convert", slArgs);
}

void magick::procReadStdOut()				// read Stdout of rsync process
{
	aOutput = mag->readAllStandardOutput();		// read byte array
/*	if ( !aOutput.isEmpty() ) {

	}*/
}

void magick::procReadStdErr()				// read Stdout of convert process
{
	aOutput = mag->readAllStandardError();		// read byte array
	if(!aOutput.isEmpty()) {
		cout << "qpictopdf - magick.cpp proc err : " << QString(aOutput).trimmed().toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
	}
	bErr = true;
}

void magick::procExit(int iD, QProcess::ExitStatus iD2)	// process has exited
{
//	disconnect(mag, SIGNAL(readyReadStandardOutput()), this, SLOT(procReadStdOut()));
	disconnect(mag, SIGNAL(readyReadStandardError()), this, SLOT(procReadStdErr()));
	disconnect(mag, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(procExit(int, QProcess::ExitStatus)));
	if(mag->exitStatus() == QProcess::NormalExit) {		// true if normal exit,  no error in Output found and not stopped by user

// **  appending 2 pics into 1
		if (iProc == 1) {
			if (iAppendIter < iAppendCnt - 1) {								// append another 2 pics into 1
				iAppendIter += 1;
				procPrep();
			} else {														// append ended, for pdf and one picture left? - copy with new name
				if (bErr == true) {
					procErr();
				} else {
					if (slToProcess.count() > (iAppendIter + 1) * 2) {		// uneven # of pics, copy n rename last
						QFile qf(slToProcess.last());
						iAppendIter += 1;
						if (qf.copy(magickAppendCreatePicName()) == true) {
							slResult.append(magickAppendCreatePicName());
							emit sigMagickAppendEnded(slResult);
						} else {											// error
							procErr();
						}
					} else {
						emit sigMagickAppendEnded(slResult);				// even # of pics
					}
				}
			}

// **  converting to pdf ended
		} else if (iProc == 2) {
			if (bErr == true) {
				procErr();
			} else {
				emit sigMagickToPdfEnded(sMOutputFile);
			}
		}

	} else {
		procErr();								// process ended with error
	}
}

void magick::procStop()						// stop rsync process on user request
{
	mag->terminate();
	QTimer::singleShot(1000, mag, SLOT(kill()));
}

void magick::procErr()						// an error has occurred
{
	if (iProc == 1) {
		slResult.clear();
		emit sigMagickAppendEnded(slResult);	// error while appending 2 pictures into 1, left to right, return emtpy result list
	} else if (iProc == 2) {
		emit sigMagickToPdfEnded("");			// error while converting (multiple) pictures into one pdf, return empty string, not file name
	}
}


#include "magick.moc"
