/****************************************************************************
 **
 **  Copyright (C) 2007-2015 by Helge Kortendieck, alan@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef QPTP_H
#define QPTP_H

#include <QApplication>
#include <QAbstractItemView>
#include <QButtonGroup>
#include <QDialog>
#include <QDir>
#include <QDirModel>
#include <QFile>
#include <QFileDialog>
#include <QFileInfo>
#include <QImage>
#include <QItemSelectionModel>
#include <QList>
#include <QMainWindow>
#include <QMenuBar>
#include <QMessageBox>
#include <QModelIndex>
#include <QPoint>
#include <QSettings>
#include <QSizePolicy>
#include <QSpinBox>
#include <QString>
#include <QStringList>
#include <QTimer>
#include <QToolBar>
#include <QTreeView>
#include <QWidget>

#include "tmpdir.h"
#include "abouthelp.h"
#include "magick.h"

#include "ui_qpictopdf.h"

//#include <cstdlib>
#include <iostream>
using namespace std;

class Qptp : public QMainWindow
{
  Q_OBJECT

public:
	Qptp(QMainWindow *parent = 0);
		~Qptp();

private:
	Ui::Qptp ui;			// GUI

	bool bD;
	bool bConH;				// concatenate true=horizontally, false=vertically
	bool bPdf;				// true=pdf, false=jpg
	bool bQuit;				// flag whether we are quitting

	double dD;
	double dD2;
	double dDpiX;
	double dDpiY;

	int iD;					// dummy int
	int iErr;				// error code: 0=no error, 1=output dir not OK, 2=nothing selected, 4=error reading images, 8=failed to create tmp dir, 
							//             16=error concatenate 2 images into 1, 32=error creating pdf file, 64=error deleting tmp dir
	int iLen;				// length of file name to suggest pdf name from
	int iRunState;			// run state: 0=cannot create, 1=can create, 2=running, can stop(kill)
	int iPicsToDo;			// number of pictures to add to pdf
	int iWShown;			// popup widget currently shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5=settings dialog)
	int iWToShow;			// popup widget to show: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5= settings dialog)
	int iX;					// cursor position for messagebox
	int iY;					// cursor position for messagebox
      
	QAction *aboutAct;
	QAction *createAct;		// create pdf
	QAction *disabledAct;
	QAction *exitAct;
	QAction *helpAct;
	QAction *refreshAct;
	QAction *stopAct;

	QDirModel *model;

	QFile f;

	QFileInfo fi;

	QImage qimg;

	QList<QModelIndex> lDirModSelIndexes; 		// list with indexes of selected pictures
	QList<QModelIndex> lDirModSelIndexesNew;	// list with new indexes of selected pictures

	QMenu *fileMenu;		// file menu
	QMenu *helpMenu;		// help menu
	QMenu *pdfMenu;			// create pdf from pictures

	QModelIndex miIndex;	// index of last dir causing expand
	QModelIndex miD;		// dummy index in model/treeview
	QModelIndex miD2;		// dummy index in model/treeview

	QPoint qPos;			// cursor position

	QString sD;				// dummy QString
	QString sD2;
	QString sDir;			// start dir for file dialogs

	QString sErr;
	QString sErr4;
	QString sErr8;
	QString sErr32;

	QString sFile;			// name of to be created output file
	QString sFileOld;		// needed to stop overwrite warning flood only check if file name changed
	QString sImgDir;		// dir with images to convert into 1 pdf
	QString sImgType;		// image type for concatenated image, taken from 1st image of selection
	QString sLastDir;		// last selected dir in treeview (from previous session)
	QString sOutDir;		// directory where to create the pdf
	QString sOutFullName;	// full path and name of output file
	QString sPath;			// full path of to be created pdf file
	QString sPdf;			// pdf name
	QString sPdfReturn;		// pdf name returned on creation
	QString sPicConc;		// full path and name of concatenated picture
	QString sSelItem0Path;	// full path and name or first selected item in tree view
	QString sSelItem0Name;	// name only of first selected item in tree view
	QString sSrcPath;		// path of last dir expanded in treeview
	QString sTmpDir;		// path to random temporary directory incl. / at end

	QStringList slAppendedPics;	// QStringList with appended pics, full path and name
	QStringList slD;
	QStringList slDel;			// files to delete
	QStringList slPicsSel;		// list of selected pictures, full path and name
	QStringList slPicsConc;		// list with concatenated pictures
	QStringList slPicsToPdf;	// list of pictures to convert into pdf, full path and name

	QToolBar *toolBar;

	aboutHelp* ahw;			// help widget
	magick* mgk;			// ImageMagick object

private slots:

	void actionsCreate();					// create actions
	void menuBarCreate();					// create menu
	void toolBarCreate();					// create toolbar
	void statusBarCreate();					// create statusbar
	void updMnTlBr();						// update menu and toolbar
	void statusBarUpd(const QString& sD);	// update statusbar
	
	void about();			// show about information
	void help();			// show help message
	void ahwCreate();		// create about/help/error dialog widget
	void ahwDel();			// hide=delete later about/help dialog widget

	void quitDelay();		// delay to allow clean up - needed ????????????????????
	void quitPrep();		// quit clicked clean up
	void quitDo();			// finally quit the program

	void clickedCreate();	// create pdf clicked

	void dmCreate();								// dir model: create
	void dmColResize(const QModelIndex miIndex);	// auto resize columns
	void dmLastDirExpand();							// expand to dir used in last session
	void dmLastDirSet(const QModelIndex miD);		// set last dir used for next session
	void dmRefresh();								// refresh file system gui
	void dmSelectionChanged(const QModelIndex miD);	// selection in tree view has changed
	void dmGetSelectedPics();						// get full path and name of pictures selected in the listview

	void delPdfAfterErr();			// delete unfinished pdf after pictures after pdf creation
	void delPicsConc();				// delete temporary concatenated images
	void delPicsOrig();				// original pictures after successful pdf creation

	bool destDirChkWri(const QString& sD);		// check if dir is writable
	void destDirEditFin();						// manually setting the pdf path finished
	void destDirGetStartDir();					// get start dir for set dir dialog
	void destDirSet(const QString& sD);			// set dest dir in line widget, home dir if invalid
	void destDirSetClicked();					// button set destination dir clicked
	QString destDirStripDir(const QString& sD);	// get path from dir or file



	void outNameSet();								// set name in output file name line edit
	void outNameDialogClicked();					// button set pdf name clicked
	void outNameLeEditFin();						// editing pdf name finished
	void outNameSetFormat();						// format pdf name
	void outNameLeFileTypeChk();					// check and switch between pdf and jpg when changing creating pdf or jpg
	QString outNamePathSlash(const QString& sD);	// ensure path has a slash at the end
	void outNameSuffix();							// ensure file name has correct suffix, jpg or pdf
	void outNameSuggest();							// suggest output file name, result.jpg or result.pdf
	void outNameSuggestToggled();					// suggest output file name check box toggled

	void outTypeToggled();				// output type toggled: pdf or jpg

	void getCursorPos();				// get cursor position - iD=x , iD2=y

	void initDelay();					// short timer to allow GUI to come up before proceeding
	void init();						// starting
	void initCont();					// continue after checking if ImageMagick's convert is accessable

	void magickCreate();								// create ImageMagick object (convert command handling)
	void magickChkReturn(int iD);						// return from checking for ImageMagick's convert command
	void magickAppend();								// concatenate 2 images into 1 left + right
	void magickAppendReturn(const QStringList& slD);	// return from concatenating 2 images into 1 left + right, list w resulting images or empty list=error
	void magickToPdf();									// convert image(s) into 1 pdf
	void magickToPdfReturn(const QString& sD);			// return  from converting image(s) into 1 pdf 

	void messageBox(int iD, const QString sD, const QString sD2);	// show QMessageBox at cursor position, 1=Info, 2=Warning, 3=Critical

	void outputGetInputFiles();		// get list of input files to process
	bool outputChkInputFiles();		// check input files: count OK, 1 or more for pdf or exactly 2 to create jpg, and readable - list has at least 1 file
	QString outputSetOutputFile();	// create full output file path & name
	bool outputChkOutputFile();		// check whether output file can be writen in destination dir and whether to overwrite file if file already exists
	void outputSetType();			// set output type, pdf or jpg
	void outputCreatePrep();		// prepare output creation

	void runStateReset();			// reset run state and update menu and toolbar

	void settingsRead();
	void settingsWrite();

	void showErr();					// show any error during pdf creation

	void tmpDirCreate();			// create tmp dir
	void tmpDirDelAfterSuccess();	// check whether the output file was created in the temporary directory, if so do not delete
	bool tmpDirDel();				// delete tmp dir

	void wCtrl(int iWToShow);	// show popup widget, if no other is shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5=settings dialog)

	signals:
		void sigAhwShow(int iD);									// show info dlg: about, help
		void sigMagickChk();										// check for ImageMagick's convert
		void sigMagickAppend(const QString&, const QStringList&, bool, bool);	// depending of appending for pdf or jpg, sTmpDir holds either temp dir or name of result jpg
		void sigMagickToPdf(const QString&, const QStringList&, const QString&, const QString&, const QString&); 
};

#endif 
